@@ -4,7 +4,7 @@ from typing import Iterable, Dict, Any, List
 
 import numpy as np
 from quick_trade.brokers import TradingClient
-import core
+from . import core
 
 
 class QuickTradeTuner(object):
@@ -13,7 +13,6 @@ class QuickTradeTuner(object):
                  tickers: Iterable,
                  intervals: Iterable,
                  starts: Iterable,
-                 strategies: List[str],
                  strategies_kwargs: Dict[str, List[Dict[str, Any]]] = None):
         """
 
@@ -21,15 +20,14 @@ class QuickTradeTuner(object):
         :param tickers: ticker
         :param intervals: list of intervals -> ['1m', '4h'...]
         :param starts: starts for client.get_data_historical (['2 Dec 2020', '3 Sep 1970'])
-        :param strategies: list of strategies -> ['strategy_supertrend'...]
-        :param strategies_kwargs: kwargs for strategies: {'strategy_supertrend': [{'multiplier': 10}]}, you can use Choise, Linspace, Range
+        :param strategies_kwargs: kwargs for strategies: {'strategy_supertrend': [{'multiplier': 10}]}, you can use Choice, Linspace, Arange as argument's value
         """
+        strategies_kwargs = core.transform_all_tunable_values(strategies_kwargs)
+        strategies = list(strategies_kwargs.keys())
         self.strategies_and_kwargs: List[str] = []
         self._strategies = []
         self._frames_data: tuple = tuple(itertools.product(tickers, intervals, starts))
         self.client = client
-        if strategies_kwargs is None:
-            strategies_kwargs = {k: [{}] for k in strategies}
         for strategy in strategies:
             for kwargs in strategies_kwargs[strategy]:
                 self._strategies.append([strategy, kwargs])
@@ -91,7 +89,7 @@ class Choise(core.TunableValue):
         self.values = values
 
 
-class Range(core.TunableValue):
+class Arange(core.TunableValue):
     def __init__(self, min_value, max_value, step):
         self.values = np.arange(min_value, max_value + step, step)
 
