@@ -39,6 +39,7 @@ from scipy import signal
 from sklearn.preprocessing import MinMaxScaler
 from tensorflow.keras.layers import Dropout, Dense, LSTM
 from tensorflow.keras.models import Sequential, load_model
+from binance.exceptions import BinanceAPIException
 
 
 class Trader(object):
@@ -57,14 +58,12 @@ class Trader(object):
     """
     profit_calculate_coef: float
     returns: utils.PREDICT_TYPE_LIST = []
-    __oldsig: utils.PREDICT_TYPE
     df: pd.DataFrame
     ticker: str
     interval: str
     __exit_order__: bool = False
     _old_predict: str = 'Exit'
     _regression_inputs: int
-    mean_diff: float
     stop_loss: float
     take_profit: float
     open_price: float
@@ -99,7 +98,6 @@ class Trader(object):
                  *args,
                  **kwargs):
         df_ = round(df, rounding)
-        self.__oldsig = utils.EXIT
         self.df = df_.reset_index(drop=True)
         self.ticker = ticker
         self.interval = interval
@@ -159,7 +157,7 @@ class Trader(object):
                       iters: int = 5,
                       plot: bool = True,
                       *args,
-                      **kwargs) -> pd.DataFrame:
+                      **kwargs) -> pd.Series:
         filtered: np.ndarray
         k_filter: KalmanFilter = KalmanFilter()
         df: pd.Series
@@ -173,14 +171,14 @@ class Trader(object):
                     name='kalman filter',
                     y=filtered.T[0],
                     line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
-        return pd.DataFrame(filtered)
+        return pd.Series(filtered.T[0])
 
     def scipy_filter(self,
                      df: pd.Series,
                      window_length: int = 101,
                      polyorder: int = 3,
                      plot: bool = True,
-                     **scipy_savgol_filter_kwargs) -> pd.DataFrame:
+                     **scipy_savgol_filter_kwargs) -> pd.Series:
         filtered = signal.savgol_filter(
             df,
             window_length=window_length,
@@ -192,7 +190,7 @@ class Trader(object):
                     name='savgol filter',
                     y=filtered,
                     line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
-        return pd.DataFrame(filtered)
+        return pd.Series(filtered)
 
     def bull_power(self, periods: int) -> np.ndarray:
         EMA = ta.trend.ema_indicator(self.df['Close'], periods)
@@ -225,7 +223,6 @@ class Trader(object):
         i: int
         for i in range(length):
             return_list.append(start + mean_diff * i)
-        self.mean_diff = mean_diff
         utils.logger.debug(f'in linear: self.mean_diff={mean_diff}')
         return np.array(return_list)
 
@@ -971,8 +968,8 @@ class Trader(object):
         self.linear = self.get_linear(self.deposit_history)
         lin_calc_df = pd.DataFrame(self.linear)
         mean_diff = float(lin_calc_df.diff().mean())
-        self.year_profit = mean_diff / self.profit_calculate_coef + money_start
-        self.year_profit = ((self.year_profit - money_start) / money_start) * 100
+        self.year_profit = mean_diff / self.profit_calculate_coef
+        self.year_profit = (self.year_profit / money_start) * 100
         self.winrate = (self.profits / self.trades) * 100
         self.info = f"""losses: {self.losses}
 trades: {self.trades}
@@ -1313,10 +1310,14 @@ winrate: {self.winrate}%"""
                          bet_for_trading_on_client: float = np.inf,
                          second_symbol_of_ticker: str = 'None',
                          rounding_bet: int = 4,
-                         coin_lotsize_division=True,
+                         coin_lotsize_division: bool = True,
+                         ignore_exceptions: bool = True,
+                         print_exc: bool = True,
                          *strategy_args,
                          **strategy_kwargs):
         """
+        :param print_exc: print binance exceptions in while loop
+        :param ignore_exceptions: ignore binance exceptions in while loop
         :param coin_lotsize_division: If for your api you specify the size of the bet in a coin, which is not in which you have a deposit, specify this parameter in the value: True. Otherwise: False, in Binance's case this is definitely the first case (True). If errors occur, try specifying the first ticker symbol instead of the second.
         :param ticker: ticker for trading.
         :param strategy: trading strategy.
@@ -1336,42 +1337,51 @@ winrate: {self.winrate}%"""
         try:
             __now__ = time.time()
             while True:
-                self.df = self.client.get_data(self.ticker, **get_data_kwargs).reset_index(drop=True)
-                strategy(*strategy_args, **strategy_kwargs)
+                try:
+                    self.df = self.client.get_data(self.ticker, **get_data_kwargs).reset_index(drop=True)
+                    strategy(*strategy_args, **strategy_kwargs)
 
-                prediction = self.get_trading_predict(
-                    trading_on_client=trading_on_client,
-                    bet_for_trading_on_client=bet_for_trading_on_client,
-                    second_symbol_of_ticker=second_symbol_of_ticker,
-                    rounding_bet=rounding_bet,
-                    coin_lotsize_division=coin_lotsize_division)
+                    prediction = self.get_trading_predict(
+                        trading_on_client=trading_on_client,
+                        bet_for_trading_on_client=bet_for_trading_on_client,
+                        second_symbol_of_ticker=second_symbol_of_ticker,
+                        rounding_bet=rounding_bet,
+                        coin_lotsize_division=coin_lotsize_division)
 
-                index = f'{self.ticker}, {time.ctime()}'
-                utils.logger.info(f"trading prediction at {index}: {prediction}")
-                if print_out:
-                    print(index, prediction)
-                self.realtime_returns[index] = prediction
-                while True:
-                    if not self.__exit_order__:
-                        price = self.client.get_ticker_price(ticker)
-                        min_ = min(self.__last_stop_loss, self.__last_take_profit)
-                        max_ = max(self.__last_stop_loss, self.__last_take_profit)
-                        if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
-                            self.__exit_order__ = True
-                            utils.logger.info('exit lot')
-                            prediction['predict'] = 'Exit'
-                            prediction['currency close'] = price
-                            index = f'{self.ticker}, {time.ctime()}'
-                            utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
-                            if print_out:
-                                print(f"trading prediction exit in sleeping at {index}: {prediction}")
-                            self.realtime_returns[index] = prediction
-                            if trading_on_client:
-                                self.client.exit_last_order()
-                    if not (time.time() < (__now__ + sleeping_time)):
-                        self._old_predict = utils.convert_signal_str(self.returns[-1])
-                        __now__ += sleeping_time
-                        break
+                    index = f'{self.ticker}, {time.ctime()}'
+                    utils.logger.info(f"trading prediction at {index}: {prediction}")
+                    if print_out:
+                        print(index, prediction)
+                    self.realtime_returns[index] = prediction
+                    while True:
+                        if not self.__exit_order__:
+                            price = self.client.get_ticker_price(ticker)
+                            min_ = min(self.__last_stop_loss, self.__last_take_profit)
+                            max_ = max(self.__last_stop_loss, self.__last_take_profit)
+                            if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
+                                self.__exit_order__ = True
+                                utils.logger.info('exit lot')
+                                prediction['predict'] = 'Exit'
+                                prediction['currency close'] = price
+                                index = f'{self.ticker}, {time.ctime()}'
+                                utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
+                                if print_out:
+                                    print(f"trading prediction exit in sleeping at {index}: {prediction}")
+                                self.realtime_returns[index] = prediction
+                                if trading_on_client:
+                                    self.client.exit_last_order()
+                        if not (time.time() < (__now__ + sleeping_time)):
+                            self._old_predict = utils.convert_signal_str(self.returns[-1])
+                            __now__ += sleeping_time
+                            break
+                except BinanceAPIException as binance_exc:
+                    if ignore_exceptions:
+                        if print_exc:
+                            print(binance_exc)
+                        utils.logger.error('error :(', exc_info=True)
+                        continue
+                    else:
+                        raise binance_exc
 
         except Exception as e:
             utils.logger.critical('error :(', exc_info=True)
@@ -1593,69 +1603,71 @@ winrate: {self.winrate}%"""
         low = self.df['Low']
         close = self.df['Close']
 
-        patterns = map(utils.ta_lib_collider_all, [
-            talib.CDL2CROWS(open_, high, low, close),
-            talib.CDL3BLACKCROWS(open_, high, low, close),
-            talib.CDL3INSIDE(open_, high, low, close),
-            talib.CDL3LINESTRIKE(open_, high, low, close),
-            talib.CDL3OUTSIDE(open_, high, low, close),
-            talib.CDL3STARSINSOUTH(open_, high, low, close),
-            talib.CDL3WHITESOLDIERS(open_, high, low, close),
-            talib.CDLABANDONEDBABY(open_, high, low, close),
-            talib.CDLADVANCEBLOCK(open_, high, low, close),
-            talib.CDLBELTHOLD(open_, high, low, close),
-            talib.CDLBREAKAWAY(open_, high, low, close),
-            talib.CDLCLOSINGMARUBOZU(open_, high, low, close),
-            talib.CDLCONCEALBABYSWALL(open_, high, low, close),
-            talib.CDLCOUNTERATTACK(open_, high, low, close),
-            talib.CDLDARKCLOUDCOVER(open_, high, low, close),
-            talib.CDLDOJI(open_, high, low, close),
-            talib.CDLDOJISTAR(open_, high, low, close),
-            talib.CDLDRAGONFLYDOJI(open_, high, low, close),
-            talib.CDLENGULFING(open_, high, low, close),
-            talib.CDLEVENINGDOJISTAR(open_, high, low, close),
-            talib.CDLEVENINGSTAR(open_, high, low, close),
-            talib.CDLGAPSIDESIDEWHITE(open_, high, low, close),
-            talib.CDLGRAVESTONEDOJI(open_, high, low, close),
-            talib.CDLHAMMER(open_, high, low, close),
-            talib.CDLHANGINGMAN(open_, high, low, close),
-            talib.CDLHARAMI(open_, high, low, close),
-            talib.CDLHARAMICROSS(open_, high, low, close),
-            talib.CDLHIGHWAVE(open_, high, low, close),
-            talib.CDLHIKKAKE(open_, high, low, close),
-            talib.CDLHIKKAKEMOD(open_, high, low, close),
-            talib.CDLHOMINGPIGEON(open_, high, low, close),
-            talib.CDLIDENTICAL3CROWS(open_, high, low, close),
-            talib.CDLINNECK(open_, high, low, close),
-            talib.CDLINVERTEDHAMMER(open_, high, low, close),
-            talib.CDLKICKING(open_, high, low, close),
-            talib.CDLKICKINGBYLENGTH(open_, high, low, close),
-            talib.CDLLADDERBOTTOM(open_, high, low, close),
-            talib.CDLLONGLEGGEDDOJI(open_, high, low, close),
-            talib.CDLLONGLINE(open_, high, low, close),
-            talib.CDLMARUBOZU(open_, high, low, close),
-            talib.CDLMATCHINGLOW(open_, high, low, close),
-            talib.CDLMATHOLD(open_, high, low, close),
-            talib.CDLMORNINGDOJISTAR(open_, high, low, close),
-            talib.CDLMORNINGSTAR(open_, high, low, close),
-            talib.CDLONNECK(open_, high, low, close),
-            talib.CDLPIERCING(open_, high, low, close),
-            talib.CDLRICKSHAWMAN(open_, high, low, close),
-            talib.CDLRISEFALL3METHODS(open_, high, low, close),
-            talib.CDLSEPARATINGLINES(open_, high, low, close),
-            talib.CDLSHOOTINGSTAR(open_, high, low, close),
-            talib.CDLSHORTLINE(open_, high, low, close),
-            talib.CDLSPINNINGTOP(open_, high, low, close),
-            talib.CDLSTALLEDPATTERN(open_, high, low, close),
-            talib.CDLSTICKSANDWICH(open_, high, low, close),
-            talib.CDLTAKURI(open_, high, low, close),
-            talib.CDLTASUKIGAP(open_, high, low, close),
-            talib.CDLTHRUSTING(open_, high, low, close),
-            talib.CDLTRISTAR(open_, high, low, close),
-            talib.CDLUNIQUE3RIVER(open_, high, low, close),
-            talib.CDLUPSIDEGAP2CROWS(open_, high, low, close),
-            talib.CDLXSIDEGAP3METHODS(open_, high, low, close)
-        ])
+        patterns = map(utils.ta_lib_collider_all,
+                       [
+                           talib.CDL2CROWS(open_, high, low, close),
+                           talib.CDL3BLACKCROWS(open_, high, low, close),
+                           talib.CDL3INSIDE(open_, high, low, close),
+                           talib.CDL3LINESTRIKE(open_, high, low, close),
+                           talib.CDL3OUTSIDE(open_, high, low, close),
+                           talib.CDL3STARSINSOUTH(open_, high, low, close),
+                           talib.CDL3WHITESOLDIERS(open_, high, low, close),
+                           talib.CDLABANDONEDBABY(open_, high, low, close),
+                           talib.CDLADVANCEBLOCK(open_, high, low, close),
+                           talib.CDLBELTHOLD(open_, high, low, close),
+                           talib.CDLBREAKAWAY(open_, high, low, close),
+                           talib.CDLCLOSINGMARUBOZU(open_, high, low, close),
+                           talib.CDLCONCEALBABYSWALL(open_, high, low, close),
+                           talib.CDLCOUNTERATTACK(open_, high, low, close),
+                           talib.CDLDARKCLOUDCOVER(open_, high, low, close),
+                           talib.CDLDOJI(open_, high, low, close),
+                           talib.CDLDOJISTAR(open_, high, low, close),
+                           talib.CDLDRAGONFLYDOJI(open_, high, low, close),
+                           talib.CDLENGULFING(open_, high, low, close),
+                           talib.CDLEVENINGDOJISTAR(open_, high, low, close),
+                           talib.CDLEVENINGSTAR(open_, high, low, close),
+                           talib.CDLGAPSIDESIDEWHITE(open_, high, low, close),
+                           talib.CDLGRAVESTONEDOJI(open_, high, low, close),
+                           talib.CDLHAMMER(open_, high, low, close),
+                           talib.CDLHANGINGMAN(open_, high, low, close),
+                           talib.CDLHARAMI(open_, high, low, close),
+                           talib.CDLHARAMICROSS(open_, high, low, close),
+                           talib.CDLHIGHWAVE(open_, high, low, close),
+                           talib.CDLHIKKAKE(open_, high, low, close),
+                           talib.CDLHIKKAKEMOD(open_, high, low, close),
+                           talib.CDLHOMINGPIGEON(open_, high, low, close),
+                           talib.CDLIDENTICAL3CROWS(open_, high, low, close),
+                           talib.CDLINNECK(open_, high, low, close),
+                           talib.CDLINVERTEDHAMMER(open_, high, low, close),
+                           talib.CDLKICKING(open_, high, low, close),
+                           talib.CDLKICKINGBYLENGTH(open_, high, low, close),
+                           talib.CDLLADDERBOTTOM(open_, high, low, close),
+                           talib.CDLLONGLEGGEDDOJI(open_, high, low, close),
+                           talib.CDLLONGLINE(open_, high, low, close),
+                           talib.CDLMARUBOZU(open_, high, low, close),
+                           talib.CDLMATCHINGLOW(open_, high, low, close),
+                           talib.CDLMATHOLD(open_, high, low, close),
+                           talib.CDLMORNINGDOJISTAR(open_, high, low, close),
+                           talib.CDLMORNINGSTAR(open_, high, low, close),
+                           talib.CDLONNECK(open_, high, low, close),
+                           talib.CDLPIERCING(open_, high, low, close),
+                           talib.CDLRICKSHAWMAN(open_, high, low, close),
+                           talib.CDLRISEFALL3METHODS(open_, high, low, close),
+                           talib.CDLSEPARATINGLINES(open_, high, low, close),
+                           talib.CDLSHOOTINGSTAR(open_, high, low, close),
+                           talib.CDLSHORTLINE(open_, high, low, close),
+                           talib.CDLSPINNINGTOP(open_, high, low, close),
+                           talib.CDLSTALLEDPATTERN(open_, high, low, close),
+                           talib.CDLSTICKSANDWICH(open_, high, low, close),
+                           talib.CDLTAKURI(open_, high, low, close),
+                           talib.CDLTASUKIGAP(open_, high, low, close),
+                           talib.CDLTHRUSTING(open_, high, low, close),
+                           talib.CDLTRISTAR(open_, high, low, close),
+                           talib.CDLUNIQUE3RIVER(open_, high, low, close),
+                           talib.CDLUPSIDEGAP2CROWS(open_, high, low, close),
+                           talib.CDLXSIDEGAP3METHODS(open_, high, low, close)
+                       ]
+                       )
 
         patterns = map(utils.anti_set_, patterns)
         self.set_open_stop_and_take()
